#include "crack.h"

/*
 * This program is copyright (c) Alec Muffett 1991 except for certain
 * portions of code ("crack-fcrypt.c") copyright (c) Robert Baldwin, Icarus
 * Sparry and Alec Muffett.  The author(s) disclaims all responsibility or
 * liability with respect to it's usage or its effect upon hardware or
 * computer systems.  This software is in the public domain and is freely
 * redistributable PROVIDED that this notice remains intact.
 */


/* THIS IS THE STRING TO MODIFY TO ADD OTHER LEADING/TRAILING CHARS */

char trailing_chars[] = "01";

/* These used to be enums. Oh well... non-portability rules OK */
#define ORDER_LCF	0
#define ORDER_LCFP	1
#define ORDER_MCF	2
#define ORDER_LCB	3
#define ORDER_LCFT	4
#define ORDER_UCF	5
#define ORDER_UCB	6
#define ORDER_MCB	7
#define NUMFILES	8

int
main (argc, argv)
    int argc;
    char *argv[];
{
    register int i;
    register char *ptr;
    int length;
    int all_digits;
    int all_letters_upper;
    int all_letters_lower;
    char buffer[STRINGSIZE];

    FILE *fp;
    FILE *fps[NUMFILES];

    if (argc == 1)
    {
	printf ("Usage:\t%s dictsrc [dictsrc ...]\n", argv[0]);
	exit (1);
    }
    for (i = 0; i < NUMFILES; i++)
    {
	char scratch[255];

	fps[i] = (FILE *) 0;
#ifndef CRACK_PLURAL		/* if not defined, skip opening it */
	if (i == ORDER_LCFP)
	{
	    continue;
	}
#endif				/* CRACK_PLURAL */
#ifndef CRACK_TRAILING		/* if not defined, skip opening it */
	if (i == ORDER_LCFT)
	{
	    continue;
	}
#endif				/* CRACK_TRAILING */
	sprintf (scratch, "dict.%c", i + 'a');
	if (!(fps[i] = fopen (scratch, "w")))
	{
	    perror (scratch);
	    exit (3);
	}
    }

    for (i = 1; argv[i]; i++)
    {
	if (!(fp = fopen (argv[i], "r")))
	{
	    perror (argv[i]);
	    exit (1);
	}
	printf ("dictpp: sourcing from file '%s'\n", argv[i]);

	while (fgets (buffer, STRINGSIZE, fp))
	{
	    Trim (buffer);

	    /* skip single letters and comments*/
	    if (!*buffer || !buffer[1] || buffer[0] == '#')
	    {
		continue;
	    }
	    length = strlen (buffer);
#ifndef CRACK_SHORT_PASSWDS
	    if (length < 5)
	    {
		continue;
	    }
#endif
	    all_letters_upper = 1;
	    all_letters_lower = 1;
	    all_digits = 1;

	    for (ptr = buffer; *ptr; ptr++)
	    {
		if (!isdigit (*ptr))
		{
		    all_digits = 0;
		}
		if (isupper (*ptr))
		{
		    all_letters_lower = 0;
		} else if (islower (*ptr))
		{
		    all_letters_upper = 0;
		}
	    }

	    if (all_digits)
	    {
		fprintf (fps[ORDER_LCF], "%.8s\n", buffer);
		continue;	/* hehehehehe */
	    }
	    if (!all_letters_upper)
	    {
		fprintf (fps[ORDER_UCF], "%.8s\n", Uppercase (buffer));
		fprintf (fps[ORDER_UCB], "%.8s\n", Uppercase (Reverse (buffer)));
	    } else
	    {
		fprintf (fps[ORDER_UCF], "%.8s\n", buffer);
		fprintf (fps[ORDER_UCB], "%.8s\n", Reverse (buffer));
	    }

	    if (!all_letters_lower)
	    {
		fprintf (fps[ORDER_LCF], "%.8s\n", Lowercase (buffer));
		fprintf (fps[ORDER_LCB], "%.8s\n", Lowercase (Reverse (buffer)));
	    } else
	    {
		fprintf (fps[ORDER_LCF], "%.8s\n", buffer);
		fprintf (fps[ORDER_LCB], "%.8s\n", Reverse (buffer));
#ifdef CRACK_PLURAL
		/* PLURALISATION */
		if (length < 8 && buffer[length - 1] != 's')
		{
		    /*
		     * Fix suggested by Jim Mattson (mattson@cs.ucsc.edu):-
		     * we should not pluralise anything more than 7 chars
		     * long, as it is a waste.
		     */
		    fprintf (fps[ORDER_LCFP], "%.7ss\n", buffer);
		}
#endif				/* CRACK_PLURAL */

#ifdef CRACK_TRAILING
		/* NUMERALISATION */
		if (length < 8)
		{
		    for (ptr = trailing_chars; *ptr; ptr++)
		    {
			fprintf (fps[ORDER_LCFT], "%.7s%c\n", buffer, *ptr);
			fprintf (fps[ORDER_LCFT], "%c%.7s\n", *ptr, buffer);
		    }
		}
#endif				/* CRACK_TRAILING */
	    }

	    if (!all_letters_lower && !all_letters_upper)
	    {
		fprintf (fps[ORDER_MCF], "%.8s\n", buffer);
		fprintf (fps[ORDER_MCB], "%.8s\n", Reverse (buffer));
	    }
	}
	fclose (fp);
    }

    for (i = 0; i < NUMFILES; i++)
    {
	if (fps[i])
	{
	    fclose (fps[i]);
	}
    }
}
